local E, L, V, P, G = unpack(select(2, ...));
local RB = E:NewModule("ReminderBuffs", "AceEvent-3.0");
local LSM = LibStub("LibSharedMedia-3.0");

E.ReminderBuffs = RB;

RB.Spell1Buffs = {
	67016, -- Flask of the North (SP)
	67017, -- Flask of the North (AP)
	67018, -- Flask of the North (STR)
	53755, -- Flask of the Frost Wyrm
	53758, -- Flask of Stoneblood
	53760, -- Flask of Endless Rage
	54212, -- Flask of Pure Mojo
	53752, -- Lesser Flask of Toughness (50 Resilience)
	17627, -- Flask of Distilled Wisdom

	33721, -- Spellpower Elixir
	53746, -- Wrath Elixir
	28497, -- Elixir of Mighty Agility
	53748, -- Elixir of Mighty Strength
	60346, -- Elixir of Lightning Speed
	60344, -- Elixir of Expertise
	60341, -- Elixir of Deadly Strikes
	60345, -- Elixir of Armor Piercing
	60340, -- Elixir of Accuracy
	53749, -- Guru's Elixir

	60343, -- Elixir of Mighty Defense
	53751, -- Elixir of Mighty Fortitude
	53764, -- Elixir of Mighty Mageblood
	60347, -- Elixir of Mighty Thoughts
	53763, -- Elixir of Protection
	53747, -- Elixir of Spirit
};

RB.Spell2Buffs = {
	57325, -- 80 AP
	57327, -- 46 SP
	57329, -- 40 Critical Strike Rating
	57332, -- 40 Haste Rating
	57334, -- 20 MP5
	57356, -- 40 Expertise Rating
	57358, -- 40 ARP
	57360, -- 40 Hit Rating
	57363, -- Tracking Humanoids
	57365, -- 40 Spirit
	57367, -- 40 AGI
	57371, -- 40 STR
	57373, -- Tracking Beasts
	57399, -- 80 AP, 46 SP
	59230, -- 40 Dodge Rating
	65247, -- 20 STR
};

RB.Spell3Buffs = {
	72588, -- Gift of the Wild
	48469, -- Mark of the Wild
};

RB.Spell4Buffs = {
	25898, -- Greater Blessing of Kings
	20217, -- Blessing of Kings
	72586, -- Blessing of Forgotten Kings
};

RB.CasterSpell5Buffs = {
	61316, -- Dalaran Brilliance
	43002, -- Arcane Brilliance
	42995, -- Arcane Intellect
};

RB.MeleeSpell5Buffs = {
	48162, -- Prayer of Fortitude
	48161, -- Power Word: Fortitude
	72590, -- Fortitude
};

RB.CasterSpell6Buffs = {
	48938, -- Greater Blessing of Wisdom
	48936, -- Blessing of Wisdom
	58777, -- Mana Spring
};

RB.MeleeSpell6Buffs = {
	48934, -- Greater Blessing of Might
	48932, -- Blessing of Might
	47436, -- Battle Shout
};

function RB:CheckFilterForActiveBuff(filter)
	local spellName, name, texture, duration, expirationTime;

	for _, spell in pairs(filter) do
		spellName = GetSpellInfo(spell);
		name, _, texture, _, _, duration, expirationTime = UnitAura("player", spellName);

		if(name) then
			return true, texture, duration, expirationTime;
		end
	end

	return false, texture, duration, expirationTime;
end

function RB:UpdateReminderTime(elapsed)
	self.expiration = self.expiration - elapsed;

	if(self.nextUpdate > 0) then
		self.nextUpdate = self.nextUpdate - elapsed;
		return;
	end

	if(self.expiration <= 0) then
		self.timer:SetText("");
		self:SetScript("OnUpdate", nil);
		return;
	end

	local timervalue, formatid;
	timervalue, formatid, self.nextUpdate = E:GetTimeInfo(self.expiration, 4);
	self.timer:SetFormattedText(("%s%s|r"):format(E.TimeColors[formatid], E.TimeFormats[formatid][1]), timervalue);
end

function RB:UpdateReminder(event, unit)
	if(event == "UNIT_AURA" and unit ~= "player") then return; end

	for i = 1, 6 do
		local hasBuff, texture, duration, expirationTime = self:CheckFilterForActiveBuff(self["Spell" .. i .. "Buffs"]);
		local button = self.frame[i];
		local reverseStyle = E.db.general.reminder.reverse

		if(hasBuff) then
			button.t:SetTexture(texture);

			if(duration == 0 and expirationTime == 0) or E.db.general.reminder.durations ~= true then
				button.t:SetAlpha(reverseStyle == true and 1 or 0.3)
				button:SetScript("OnUpdate", nil);
				button.timer:SetText(nil);
				CooldownFrame_SetTimer(button.cd, 0, 0, 0);
			else
				button.expiration = expirationTime - GetTime();
				button.nextUpdate = 0;
				button.t:SetAlpha(1)
				CooldownFrame_SetTimer(button.cd, expirationTime - duration, duration, 1);
				button.cd:SetReverse(reverseStyle == true and true or false)
				button:SetScript("OnUpdate", self.UpdateReminderTime);
			end
		else
			CooldownFrame_SetTimer(button.cd, 0, 0, 0);
			button.t:SetAlpha(reverseStyle == true and 0.3 or 1)
			button:SetScript("OnUpdate", nil);
			button.timer:SetText(nil);
			button.t:SetTexture(self.DefaultIcons[i]);
		end
	end
end

function RB:CreateButton()
	local button = CreateFrame("Button", nil, ElvUI_ReminderBuffs);
	button:SetTemplate("Default");

	button.t = button:CreateTexture(nil, "OVERLAY");
	button.t:SetTexCoord(unpack(E.TexCoords));
	button.t:SetInside();
	button.t:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");

	button.cd = CreateFrame("Cooldown", nil, button, "CooldownFrameTemplate");
	button.cd:SetInside();
	button.cd.noOCC = true;
	button.cd.noCooldownCount = true;

	button.timer = button.cd:CreateFontString(nil, "OVERLAY");
	button.timer:SetPoint("CENTER");

	return button;
end

function RB:EnableRB()
	ElvUI_ReminderBuffs:Show();
	self:RegisterEvent("UNIT_AURA", "UpdateReminder");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "UpdateReminder");
	self:RegisterEvent("CHARACTER_POINTS_CHANGED", "UpdateReminder");
	E.RegisterCallback(self, "RoleChanged", "UpdateSettings");
	self:UpdateReminder();
end

function RB:DisableRB()
	ElvUI_ReminderBuffs:Hide();
	self:UnregisterEvent("UNIT_AURA");
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	self:UnregisterEvent("CHARACTER_POINTS_CHANGED");
	E.UnregisterCallback(self, "RoleChanged", "UpdateSettings");
end

function RB:UpdateSettings(isCallback)
	local frame = self.frame;
	frame:Width(E.RBRWidth);

	self:UpdateDefaultIcons();

	for i = 1, 6 do
		local button = frame[i];
		button:ClearAllPoints();
		button:SetWidth(E.RBRWidth);
		button:SetHeight(E.RBRWidth);

		if(i == 1) then
			button:Point("TOP", ElvUI_ReminderBuffs, "TOP", 0, 0);
		else
			button:Point("TOP", frame[i - 1], "BOTTOM", 0, E.Border - E.Spacing*3);
		end

		if(i == 6) then
			button:Point("BOTTOM", ElvUI_ReminderBuffs, "BOTTOM", 0, 0);
		end

		if(E.db.general.reminder.durations) then
			button.cd:SetAlpha(1);
		else
			button.cd:SetAlpha(0);
		end

		local font = LSM:Fetch("font", E.db.general.reminder.font);
		button.timer:FontTemplate(font, E.db.general.reminder.fontSize, E.db.general.reminder.fontOutline);
	end

	if(not isCallback) then
		if(E.db.general.reminder.enable) then
			RB:EnableRB();
		else
			RB:DisableRB();
		end
	else
		self:UpdateReminder();
	end
end

function RB:UpdatePosition()
	Minimap:ClearAllPoints();
	ElvConfigToggle:ClearAllPoints();
	ElvUI_ReminderBuffs:ClearAllPoints();
	if(E.db.general.reminder.position == "LEFT") then
		Minimap:Point("TOPRIGHT", MMHolder, "TOPRIGHT", -E.Border, -E.Border);
		ElvConfigToggle:SetPoint("TOPRIGHT", LeftMiniPanel, "TOPLEFT", E.Border - E.Spacing*3, 0);
		ElvConfigToggle:SetPoint("BOTTOMRIGHT", LeftMiniPanel, "BOTTOMLEFT", E.Border - E.Spacing*3, 0);
		ElvUI_ReminderBuffs:SetPoint("TOPRIGHT", Minimap.backdrop, "TOPLEFT", E.Border - E.Spacing*3, 0);
		ElvUI_ReminderBuffs:SetPoint("BOTTOMRIGHT", Minimap.backdrop, "BOTTOMLEFT", E.Border - E.Spacing*3, 0);
	else
		Minimap:Point("TOPLEFT", MMHolder, "TOPLEFT", E.Border, -E.Border);
		ElvConfigToggle:SetPoint("TOPLEFT", RightMiniPanel, "TOPRIGHT", -E.Border + E.Spacing*3, 0);
		ElvConfigToggle:SetPoint("BOTTOMLEFT", RightMiniPanel, "BOTTOMRIGHT", -E.Border + E.Spacing*3, 0);
		ElvUI_ReminderBuffs:SetPoint("TOPLEFT", Minimap.backdrop, "TOPRIGHT", -E.Border + E.Spacing*3, 0);
		ElvUI_ReminderBuffs:SetPoint("BOTTOMLEFT", Minimap.backdrop, "BOTTOMRIGHT", -E.Border + E.Spacing*3, 0);
	end
end

function RB:UpdateDefaultIcons()
	self.DefaultIcons = {
		[1] = "Interface\\Icons\\INV_Potion_97",
		[2] = "Interface\\Icons\\Spell_Misc_Food",
		[3] = "Interface\\Icons\\Spell_Nature_Regeneration",
		[4] = "Interface\\Icons\\Spell_Magic_GreaterBlessingofKings",
		[5] = (E.Role == "Caster" and "Interface\\Icons\\Spell_Holy_MagicalSentry") or "Interface\\Icons\\Spell_Holy_WordFortitude",
		[6] = (E.Role == "Caster" and "Interface\\Icons\\Spell_Holy_GreaterBlessingofWisdom") or "Interface\\Icons\\Ability_Warrior_BattleShout"
	};

	if(E.Role == "Caster") then
		self.Spell5Buffs = self.CasterSpell5Buffs;
		self.Spell6Buffs = self.CasterSpell6Buffs;
	else
		self.Spell5Buffs = self.MeleeSpell5Buffs;
		self.Spell6Buffs = self.MeleeSpell6Buffs;
	end
end

function RB:Initialize()
	if(not E.private.general.minimap.enable) then return end

	self.db = E.db.general.reminder;

	local frame = CreateFrame("Frame", "ElvUI_ReminderBuffs", Minimap);
	frame:Width(E.RBRWidth);
	if(E.db.general.reminder.position == "LEFT") then
		frame:Point("TOPRIGHT", Minimap.backdrop, "TOPLEFT", E.Border - E.Spacing*3, 0);
		frame:Point("BOTTOMRIGHT", Minimap.backdrop, "BOTTOMLEFT", E.Border - E.Spacing*3, 0);
	else
		frame:Point("TOPLEFT", Minimap.backdrop, "TOPRIGHT", -E.Border + E.Spacing*3, 0);
		frame:Point("BOTTOMLEFT", Minimap.backdrop, "BOTTOMRIGHT", -E.Border + E.Spacing*3, 0);
	end
	self.frame = frame;

	for i = 1, 6 do
		frame[i] = self:CreateButton();
		frame[i]:SetID(i);
	end

	self:UpdateSettings();
end

local function InitializeCallback()
	RB:Initialize()
end

E:RegisterModule(RB:GetName(), InitializeCallback)